
QuestMods = CreateFrame("Frame", "QuestModsMainFrame");

QuestMods.Player = UnitName("player");
QuestMods.Realm = GetRealmName();

QuestMods.Silent = false;
QuestMods.Moving = false;
QuestMods.Login = 1;

local QuestList = {};
local QuestIDs = {};
local Headers = {};

local ObjList = {};
local MsgList = {};
local QIStage = {};
local QIList = {};

local IsFlashing = false;
local FlashFlag = false;
local FlashCount = 0;

local QuestSoundTime = 0;

local LastMsgInterval = .2;
local TimeSinceLastMsg = LastMsgInterval;

local LastFlashInterval = .25;
local TimeSinceLastFlash = LastFlashInterval;

local OldQuestLogTitleButtonOnClick;

local StartTime = 0;

function QuestMods.Init()
	QuestMods:SetScript("OnEvent", QuestMods.OnEvent);
	QuestMods:SetScript("OnUpdate", QuestMods.OnUpdate);
	
	QuestMods:RegisterEvent("PLAYER_LOGIN");
	QuestMods:RegisterEvent("UI_INFO_MESSAGE");
	QuestMods:RegisterEvent("QUEST_WATCH_UPDATE");
	QuestMods:RegisterEvent("QUEST_LOG_UPDATE");
	QuestMods:RegisterEvent("UNIT_QUEST_LOG_CHANGED");
end

function QuestMods.OnEvent(self, event, ...)
	local arg1 = ...;
	
	if (event == "PLAYER_LOGIN") then
		-- We will be handling the expanding, thank you very much...
		ExpandQuestHeader(0);
		
		QuestMods.OptionsSetup();
		
		-- Hooks for quest log
		QuestLog_Update = QuestMods.QuestLogUpdate;
		QuestLogScrollFrame.update = QuestMods.QuestLogUpdate;
		
		OldQuestLogTitleButtonOnClick = QuestLogTitleButton_OnClick;
		QuestLogTitleButton_OnClick = QuestMods.QuestLogTitleButtonOnClick;
		
		-- Track watches
		hooksecurefunc("AddQuestWatch", QuestMods.AddQuestWatch);
		hooksecurefunc("RemoveQuestWatch", QuestMods.RemoveQuestWatch);
		
		hooksecurefunc("CollapseQuestHeader", function(questIndex) ExpandQuestHeader(questIndex); end);
		hooksecurefunc("QuestLog_SetSelection", QuestMods.QuestLogSetSelection);
		
		-- Levels on watches and map
		hooksecurefunc("WorldMapFrame_UpdateQuests", QuestMods.WorldMapFrameUpdateQuests);
		hooksecurefunc("WatchFrame_Update", QuestMods.WatchFrameUpdate);
		
		-- Get rid of pointless accept window on autoquest
		--ShowQuestOffer = QuestMods.AutoQuestButtonOnClick;
		QuestFrameDetailPanel:HookScript("OnShow", QuestMods.QuestFrameDetailOnShow);
		
		--if (QuestMods_SavedVars.Login == "Y") then
		--	QuestMods.CheckForNewItem();
		--else
		--	QuestMods.CheckBags();
		--end
		
		--QuestMods.Silent = false;
		
		-- Register late to ignore pointless prelogin processing
		QuestMods:RegisterEvent("BAG_UPDATE");
		
		if (QuestMods_SavedVars.Login ~= "Y") then
			StartTime = GetTime();
		end
	end
	
	if (event == "UI_INFO_MESSAGE" and (arg1 ~= gsub(arg1, "(.*): %d+/%d+", "%1", 1) or arg1 ~= gsub(arg1, "(.*) %("..COMPLETE.."%)", "%1", 1))) then
		--if (QuestMods_SavedVars.Party == "Y" and GetNumPartyMembers() > 0) then
		--	QuestMods.SendToParty("ALERT", "objtext", QuestMods.Player.." - "..arg1);
		--end
		
		if (QuestMods_SavedVars.Chat == "Y") then
			print("|cff00ffff"..arg1);
		end
		
		if (IsInGroup() and QuestMods_SavedVars.Party == "Y") then
			QuestMods.SendToGroupChat(arg1);
		end
	end
	
	if (event == "QUEST_WATCH_UPDATE") then
		--QuestMods.Print("QUEST_WATCH_UPDATE - arg1: "..(arg1 or "nil"));
		
		if (arg1 and arg1 > 0) then
			ObjList[arg1] = true;
		end
	end
	
	if (event == "QUEST_LOG_UPDATE" or (event == "UNIT_QUEST_LOG_CHANGED" and arg1 == "player")) then
		QuestMods.CheckCompletions();
		QuestMods.CheckForNewItem();
	end
	
	if (event == "BAG_UPDATE" and arg1 >=0 and arg1 <= NUM_BAG_SLOTS) then
		QuestMods.CheckForNewItem();
	end
end

function QuestMods.OnUpdate(self, elapsed)
	TimeSinceLastMsg = TimeSinceLastMsg + elapsed;
	
	if (TimeSinceLastMsg > LastMsgInterval) then
		if (#MsgList > 0 and not QuestMods_MessageFrame.animFade:IsPlaying()) then
			QuestMods_MessageFrame.text:SetText(MsgList[1]);
			tremove(MsgList, 1);
			QuestMods_MessageFrame.animFade:Play();
		end
		
		if (QuestSoundTime ~= 0 and GetTime() > QuestSoundTime) then
			QuestSoundTime = 0;
		end

		TimeSinceLastMsg = 0;
	end
	
	TimeSinceLastFlash = TimeSinceLastFlash + elapsed;
	
	if (TimeSinceLastFlash > LastFlashInterval) then
		if (IsFlashing) then
			if (FlashFlag) then
				QuestMods_QuestIconFrame.flashTexture:Show();
			else
				QuestMods_QuestIconFrame.flashTexture:Hide();
				FlashCount = FlashCount + 1;
				
				if (FlashCount == 9) then
					IsFlashing = false;
				end
			end
			
			FlashFlag = not FlashFlag;
		end
		
		TimeSinceLastFlash = 0;
	end
end

function QuestMods.QueueMessage(text)
	tinsert(MsgList, text);
end

function QuestMods.CheckCompletions()
	for k, v in pairs(ObjList) do
		local complete = true;
		
		for i = 1, GetNumQuestLeaderBoards(k) do
			local desc, type, done = GetQuestLogLeaderBoard(i, k)
			
			if (not done) then
				complete = false;
				break;
			end
		end
		
		if (complete) then
			local name = GetQuestLogTitle(k);
			
			--UIErrorsFrame:AddMessage(GREEN_FONT_COLOR_CODE..format(QUESTMODS_QUESTCOMPLETE, name));
			QuestMods.QueueMessage(format(QUESTMODS_QUESTCOMPLETE, name));
			print("|cff00ff00"..format(QUESTMODS_QUESTCOMPLETE, name));
			
			if (IsInGroup() and QuestMods_SavedVars.Party == "Y") then
				QuestMods.SendToGroupChat(format(QUESTMODS_QUESTCOMPLETE, name));
			end
			
			if (QuestMods_SavedVars.QuestSound == "Y" and QuestSoundTime == 0) then
				PlaySoundFile("Interface\\Addons\\QuestMods\\Sounds\\QuestObjComplete.mp3");
				QuestSoundTime = GetTime() + 3;
			end
			
			--if (QuestMods_SavedVars.Party == "Y" and GetNumPartyMembers() > 0) then
			--	QuestMods.SendToParty("ALERT", "complete", QuestMods.Player.." - Objectives completed: "..name);
			--end
		end
	end
	
	wipe(ObjList);
end

function QuestMods.SendToGroupChat(msg)
	SendChatMessage(msg, (IsInGroup(LE_PARTY_CATRGORY_INSTANCE) and "INSTANCE_CHAT") or "PARTY");
end

function QuestMods.AutoQuestButtonOnClick(questIndex)
	QuestLog_OpenToQuest(questIndex);
end

function QuestMods.QuestFrameDetailOnShow()
	if (QuestGetAutoAccept()) then
		QuestFrameAcceptButton:SetText(CLOSE);
		QuestFrameCloseButton:Enable();
	else
		QuestFrameAcceptButton:SetText(ACCEPT);
	end
end

function QuestMods.CheckBags()
	local count = 0;
	local found = true;
	
	wipe(QIStage);
	
	for i = 0, NUM_BAG_SLOTS do
		for j = 1, GetContainerNumSlots(i) do
			local _, questID, IsActive = GetContainerItemQuestInfo(i, j);
			
			if (questID) then
				if (not QIStage[questID]) then
					QIStage[questID] = true;
				end
				
				--if (not IsActive) then
				--	IsActive = IsQuestFlaggedCompleted(questID);
				--end
				
				if (not IsActive) then
					if (QIList[questID] == nil) then
						found = false;
					end
					
					count = count + 1;
				end
			end
		end
	end
	
	local temp = QIList;
	QIList = QIStage;
	QIStage = temp;
	
	return ((count < 1 and 1) or (not found and 2) or 0);
end

function QuestMods.CheckForNewItem()
	local check = QuestMods.CheckBags();
	
	if (check == 2) then
		if (StartTime == 0 or GetTime() - StartTime > 3) then
			QuestMods_QuestIconFrame:Hide();
			QuestMods_QuestIconFrame:Show();
			StartTime = 0;
		end
	elseif (check == 1 and not QuestMods.Moving) then
		QuestMods_QuestIconFrame:Hide();
	end
end

function QuestMods.QuestIconFrameOnShow(self)
	if (not QuestMods.Silent) then
		IsFlashing = true;
		FlashFlag = true;
		FlashCount = 0;
		
		if (QuestMods_QuestIconFrame.text.animFade:IsPlaying()) then
			QuestMods_QuestIconFrame.text.animFade:Stop();
		end
		
		QuestMods_QuestIconFrame.text.animFade:Play();
		
		if (QuestMods_SavedVars.QISound == "Y") then
			PlaySoundFile("Interface\\Addons\\QuestMods\\Sounds\\QuestItem.mp3");
		end
	end
end

function QuestMods.QuestIconFrameOnHide(self)
	IsFlashing = false;
	
	if (self.text.animFade:IsPlaying()) then
		self.text.animFade:Stop();
		QuestMods_QuestIconFrame.text:Hide();
	end
	
	self.flashTexture:Hide();
end

function QuestMods.GetQuests()
	local isCollapsed;
	
	wipe(QuestList);
	wipe(QuestIDs);
	wipe(Headers);
	
	for questIndex = 1, GetNumQuestLogEntries() do
		local title, _, _, _, isHeader, _, _, _, questID = GetQuestLogTitle(questIndex);
		
		if (isHeader) then
			isCollapsed = QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Collapsed[title];
			Headers[title] = true;
		else
			QuestIDs[questID] = true;
		end
		
		if (isHeader or not isCollapsed) then
			tinsert(QuestList, questIndex);
		end
	end
end

local function _QuestLog_HighlightQuest(questLogTitle)
	local prevParent = QuestLogHighlightFrame:GetParent();
	if ( prevParent and prevParent ~= questLogTitle ) then
		-- set prev quest's colors back to normal
		local prevName = prevParent:GetName();
		prevParent:UnlockHighlight();
		prevParent.tag:SetTextColor(prevParent.r, prevParent.g, prevParent.b);
		prevParent.groupMates:SetTextColor(prevParent.r, prevParent.g, prevParent.b);
	end
	if ( questLogTitle ) then
		local name = questLogTitle:GetName();
		-- highlight the quest's colors
		questLogTitle.tag:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		questLogTitle.groupMates:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
		questLogTitle:LockHighlight();
		-- reposition highlight frames
		QuestLogHighlightFrame:SetParent(questLogTitle);
		QuestLogHighlightFrame:SetPoint("TOPLEFT", questLogTitle, "TOPLEFT", 0, 0);
		QuestLogHighlightFrame:SetPoint("BOTTOMRIGHT", questLogTitle, "BOTTOMRIGHT", 0, 0);
		QuestLogSkillHighlight:SetVertexColor(questLogTitle.r, questLogTitle.g, questLogTitle.b);
		QuestLogHighlightFrame:Show();
	else
		QuestLogHighlightFrame:Hide();
	end
end

function QuestMods.QuestLogUpdate()
	if (QuestMods.Login) then
		if (QuestMods.Login == 2) then
			QuestMods.SetCurrentWatches();
			QuestMods.Login = false;
		else
			QuestMods.Login = 2;
		end
	end
	
	if ( not QuestLogFrame:IsShown() ) then
		return;
	end
	
	local numEntries, numQuests = GetNumQuestLogEntries();
	if ( numEntries == 0 ) then
		HideUIPanel(QuestLogDetailFrame);
		QuestLogDetailFrame.timeLeft = nil;
		EmptyQuestLogFrame:Show();
		QuestLog_SetSelection(0);
	else
		EmptyQuestLogFrame:Hide();
	end

	QuestLog_UpdateMapButton();
	
	-- Update Quest Count
	QuestLog_UpdateQuestCount(numQuests);

	-- If no selection then set it to the first available quest
	local questLogSelection = GetQuestLogSelection();
	local haveSelection = questLogSelection ~= 0;
	if ( numQuests > 0 and not haveSelection ) then
		if ( QuestLogFrame.selectedIndex ) then
			QuestLog_SetNearestValidSelection();
		else
			QuestLog_SetFirstValidSelection();
		end
		questLogSelection = GetQuestLogSelection();
	end
	QuestLogFrame.selectedIndex = questLogSelection;
    
    --The counts may have changed with SetNearestValidSelection expanding quest headers.
    --Bug ID 170644
    numEntries, numQuests = GetNumQuestLogEntries();

	-- hide the details if we don't have a selected quest
	if ( not haveSelection ) then
		HideUIPanel(QuestLogDetailFrame);
	end

	-- update the group timer
	local haveGroup = IsInGroup();
	if ( haveGroup ) then
		QuestLogFrame.groupUpdateTimer = 0;
	else
		QuestLogFrame.groupUpdateTimer = nil;
	end

	-- hide the highlight frame initially, it may be shown when we loop through the quest listing if a quest is selected
	QuestLogHighlightFrame:Hide();

	-- Update the quest listing
	local buttons = QuestLogScrollFrame.buttons;
	local numButtons = #buttons;
	local scrollOffset = HybridScrollFrame_GetOffset(QuestLogScrollFrame);
	local buttonHeight = buttons[1]:GetHeight();
	local displayedHeight = 0;

	local numPartyMembers = GetNumSubgroupMembers();
	local questIndex, questLogTitle, questTitleTag, questNumGroupMates, questNormalText, questCheck;
	local title, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID, startEvent, displayQuestID;
	local color;
	local partyMembersOnQuest, tempWidth, textWidth;
	
	local index; 
	
	QuestMods.GetQuests();
	numEntries = #QuestList;
	
	for i=1, numButtons do
		questLogTitle = buttons[i];
		index = i + scrollOffset;
		
		if ( index <= numEntries ) then
			questIndex = QuestList[index];
			questLogTitle:SetID(questIndex);
			questTitleTag = questLogTitle.tag;
			questNumGroupMates = questLogTitle.groupMates;
			questCheck = questLogTitle.check;
			questNormalText = questLogTitle.normalText;
			
			title, level, questTag, suggestedGroup, isHeader, _, isComplete, isDaily, questID, startEvent, displayQuestID = GetQuestLogTitle(questIndex);

			if ( isHeader ) then
				-- set the title
				if ( title ) then
					questLogTitle:SetText(title);
				else
					questLogTitle:SetText("");
				end

				isCollapsed = QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Collapsed[title];
				
				-- set the normal texture based on the header's collapsed state
				if ( isCollapsed ) then
					questLogTitle:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
				else
					questLogTitle:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up"); 
				end
				questLogTitle:SetHighlightTexture("Interface\\Buttons\\UI-PlusButton-Hilight");

				questNumGroupMates:Hide();
				questTitleTag:Hide();
				questCheck:Hide();
				questLogTitle.QuestionMark:Hide();
			else
				-- set the title
				if ( ENABLE_COLORBLIND_MODE == "1" or QuestMods_SavedVars.Levels == "Y" ) then
					title = "["..level.."] " .. title;
				end
				if (questID and displayQuestID) then
					questLogTitle:SetText("  "..questID.." - "..title);
				else
					questLogTitle:SetText("  "..title);
				end

				-- this isn't a header, hide the header textures
				questLogTitle:SetNormalTexture("");
				questLogTitle:SetHighlightTexture("");

				-- If not a header see if any nearby group mates are on this quest
				partyMembersOnQuest = 0;
				for j=1, numPartyMembers do
					if ( IsUnitOnQuest(questIndex, "party"..j) ) then
						partyMembersOnQuest = partyMembersOnQuest + 1;
					end
				end
				if ( partyMembersOnQuest > 0 ) then
					questNumGroupMates:SetText("["..partyMembersOnQuest.."]");
					questNumGroupMates:Show();
				else
					questNumGroupMates:Hide();
				end

				-- figure out which tag to show, if any
				if ( isComplete and isComplete < 0 ) then
					questTag = FAILED;
				elseif ( isComplete and isComplete > 0 ) then
					questTag = COMPLETE;
				elseif ( isDaily ) then
					if ( questTag ) then
						questTag = format(DAILY_QUEST_TAG_TEMPLATE, questTag);
					else
						questTag = DAILY;
					end
				end
				if ( questTag ) then
					questTitleTag:SetText("("..questTag..")");
					questTitleTag:Show();
				else
					questTitleTag:Hide();
				end

				-- show the quest check if the quest is being watched
				if (IsQuestWatched(questIndex) and not QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Watched[questID]) then
					QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Watched[questID] = true;
				end
				
				if ( QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Watched[questID] ) then
					questCheck:Show();
				else
					questCheck:Hide();
				end
				
				-- Show the question mark icon for auto-complete quests
				if (isComplete and isComplete>0 and GetQuestLogIsAutoComplete(questIndex)) then
					questLogTitle.QuestionMark:Show();
				else
					questLogTitle.QuestionMark:Hide();
				end
			end
			
			-- Save if its a header or not
			questLogTitle.isHeader = isHeader;

			-- resize the title button so everything fits where it's supposed to
			QuestLogTitleButton_Resize(questLogTitle);

			-- Color the quest title and highlight according to the difficulty level
			if ( isHeader ) then
				color = QuestDifficultyColors["header"];
			else
				color = GetQuestDifficultyColor(level);
			end
			questTitleTag:SetTextColor(color.r, color.g, color.b);
			questLogTitle:SetNormalFontObject(color.font);
			questNumGroupMates:SetTextColor(color.r, color.g, color.b);
			questLogTitle.r = color.r;
			questLogTitle.g = color.g;
			questLogTitle.b = color.b;
			questLogTitle:Show();

			-- Place the highlight and lock the highlight state
			if ( questLogSelection == questIndex ) then
				_QuestLog_HighlightQuest(questLogTitle);
			else
				questLogTitle:UnlockHighlight();
			end
		else
			questLogTitle:Hide();
		end
		displayedHeight = displayedHeight + buttonHeight;
	end
	HybridScrollFrame_Update(QuestLogScrollFrame, numEntries * buttonHeight, displayedHeight);
	
	local selectedIsComplete = select(7, GetQuestLogTitle(questLogSelection));
	if (selectedIsComplete and GetQuestLogIsAutoComplete()) then
		QuestLogFrameCompleteButton:Show();
	else
		QuestLogFrameCompleteButton:Hide();
	end
	
	-- update the control panel
	QuestLogControlPanel_UpdateState();
	
	QuestMods.CleanLists();
end

function QuestMods.CleanLists()
	local tbl = QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Watched;
	
	for k, v in pairs(tbl) do
		if (not QuestIDs[k]) then
			tbl[k] = nil;
		end
	end
	
	tbl = QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Collapsed;
	
	for k, v in pairs(tbl) do
		if (not Headers[k]) then
			tbl[k] = nil;
		end
	end
end

function QuestMods.QuestLogTitleButtonOnClick(self, button)
	local questIndex = self:GetID();
	
	if (IsControlKeyDown()) then
		QuestLog_SetSelection(questIndex);
		SetAbandonQuest();
		
		if (not IsShiftKeyDown()) then
			local items = GetAbandonQuestItems();
			
			if ( items ) then
				StaticPopup_Hide("ABANDON_QUEST");
				StaticPopup_Show("ABANDON_QUEST_WITH_ITEMS", GetAbandonQuestName(), items);
			else
				StaticPopup_Hide("ABANDON_QUEST_WITH_ITEMS");
				StaticPopup_Show("ABANDON_QUEST", GetAbandonQuestName());
			end
		else
			AbandonQuest();
			PlaySound("igQuestLogAbandonQuest");
		end
		
		return;
	end
	
	OldQuestLogTitleButtonOnClick(self, button);
end

function QuestMods.QuestLogSetSelection(questIndex)
	local title, _, _, _, isHeader = GetQuestLogTitle(questIndex);
	
	if ( isHeader ) then
		local tbl = QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Collapsed;
		
		if (tbl[title]) then
			tbl[title] = nil;
		else
			tbl[title] = true;
		end
	end
	
	QuestMods.QuestLogUpdate()
end

function QuestMods.SetCurrentWatches()
	for questIndex = 1, GetNumQuestLogEntries() do
		local questID = QuestMods.GetQuestID(questIndex);
		
		if (QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Watched[questID] and not IsQuestWatched(questIndex)) then
			AddQuestWatch(questIndex);
		end
	end
end

function QuestMods.AddQuestWatch(questIndex)
	local questId = QuestMods.GetQuestID(questIndex);
	
	if (questId) then
		QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Watched[questId] = true;
	end
end

function QuestMods.RemoveQuestWatch(questIndex)
	QuestMods_SavedVars[QuestMods.Realm][QuestMods.Player].Watched[QuestMods.GetQuestID(questIndex)] = nil;
end

function QuestMods.GetQuestID(questIndex)
	if (questIndex < 1 or questIndex > GetNumQuestLogEntries()) then
		return;
	end
	
	local _, _, _, _, _, _, _, _, questID = GetQuestLogTitle(questIndex);
	
	return questID;
end

function QuestMods.WatchFrameUpdate()
	local title, level, isComplete, line;
	
	for i, button in ipairs(WATCHFRAME_LINKBUTTONS) do
		if (button.type == "QUEST") then
			local index = GetQuestIndexForWatch(button.index);
			
			if (index) then
				title, level, _, _, _, _, isComplete = GetQuestLogTitle(index);
				
				if (QuestMods_SavedVars.Levels == "Y") then
					title = "["..level.."] "..title;
				end
				
				line = button.lines[button.startLine];
				line.text:SetText(title);
				
				if ( line.text:GetHeight() > WATCHFRAME_LINEHEIGHT ) then
					if ( isComplete ) then
						line:SetHeight(line.text:GetHeight() + 4);
					else
						line:SetHeight(WATCHFRAME_MULTIPLE_LINEHEIGHT);
						line.text:SetHeight(WATCHFRAME_MULTIPLE_LINEHEIGHT);
					end
				end
			end
		end
	end
end

function QuestMods.WorldMapFrameUpdateQuests()
	if (QuestMods_SavedVars.Levels ~= "Y") then
		return;
	end
	
	local title, level;
	local questFrame;

	for i = 1, MAX_NUM_QUESTS do
		questFrame = _G["WorldMapQuestFrame"..i];
		
		if (questFrame and questFrame.questLogIndex and questFrame.questLogIndex > 0) then
			title, level = GetQuestLogTitle(questFrame.questLogIndex);
			
			if (title and strlen(title) > 0) then
				questFrame.title:SetText("["..level.."] "..title);
				questFrame:SetHeight(max(questFrame.title:GetHeight() + questFrame.objectives:GetHeight() + QUESTFRAME_PADDING, QUESTFRAME_MINHEIGHT));
			end
		end
	end
end

function QuestMods.ShowTitle()
	local questTitle;
	if ( QuestInfoFrame.questLog ) then
		questTitle, level = GetQuestLogTitle(GetQuestLogSelection());
		if ( not questTitle ) then
			questTitle = "";
		elseif (QuestMods_SavedVars.Levels == "Y") then
			questTitle = "["..level.."] "..questTitle;
		end
		if ( IsCurrentQuestFailed() ) then
			questTitle = questTitle.." - ("..FAILED..")";
		end
	else
		questTitle = GetTitleText();
	end
	QuestInfoTitleHeader:SetText(questTitle);
	return QuestInfoTitleHeader;
end

QUEST_TEMPLATE_LOG = { questLog = true, chooseItems = nil, tooltip = "GameTooltip",
	elements = {
		QuestMods.ShowTitle, 5, -5,
		QuestInfo_ShowObjectivesText, 0, -5,
		QuestInfo_ShowTimer, 0, -10,
		QuestInfo_ShowObjectives, 0, -10,
		QuestInfo_ShowSpecialObjectives, 0, -10,
		QuestInfo_ShowRequiredMoney, 0, 0,
		QuestInfo_ShowGroupSize, 0, -10,
		QuestInfo_ShowDescriptionHeader, 0, -10,
		QuestInfo_ShowDescriptionText, 0, -5,
		QuestInfo_ShowRewards, 0, -10,
		QuestInfo_ShowSpacer, 0, -10
	}
}

QUEST_TEMPLATE_MAP1 = { questLog = true, chooseItems = nil, fadingText = nil, tooltip = nil,
	elements = {
		QuestMods.ShowTitle, 30, -10,
		QuestInfo_ShowObjectivesText, 0, -5,
		QuestInfo_ShowDescriptionHeader, 0, -10,
		QuestInfo_ShowDescriptionText, 0, -5,
		QuestInfo_ShowSpacer, 0, -10
	}
}

QuestMods.Init();

